from afeng_tools.fastapi_tool.common.po_service.article_po_service_ import ArticlePoService
from afeng_tools.fastapi_tool.common.po_service.blacklist_po_service_ import BlacklistPoService
from afeng_tools.fastapi_tool.common.po_service.category_po_service_ import CategoryPoService
from afeng_tools.fastapi_tool.common.po_service.group_po_service_ import GroupPoService
from afeng_tools.fastapi_tool.common.po_service.link_po_service_ import LinkPoService
from afeng_tools.fastapi_tool.common.po_service.resource_po_service_ import ResourcePoService
from afeng_tools.fastapi_tool.common.po_service.sitemap_po_service_ import SitemapPoService
from afeng_tools.fastapi_tool.common.po_service.tag_po_service_ import TagPoService


class CommonPoService:
    """通用po服务"""

    def __init__(self, db_code: str):
        self.db_code = db_code
        self.article_po_service = ArticlePoService(self.db_code)
        self.blacklist_po_service = BlacklistPoService(self.db_code)
        self.category_po_service = CategoryPoService(self.db_code)
        self.group_po_service = GroupPoService(self.db_code)
        self.link_po_service = LinkPoService(self.db_code)
        self.resource_po_service = ResourcePoService(self.db_code)
        self.sitemap_po_service = SitemapPoService(self.db_code)
        self.tag_po_service = TagPoService(self.db_code)





"""
sqlalchemy数据库复制工具
"""
from typing import Callable, Any, Type
from sqlalchemy.orm import Session

from afeng_tools.sqlalchemy_tool.core import sqlalchemy_model_utils
from afeng_tools.sqlalchemy_tool.core.sqlalchemy_db_tools import SqlalchemyDbTool
from afeng_tools.sqlalchemy_tool.core.sqlalchemy_item import DatabaseInfoItem


def create_db_tool(database_uri: str):
    return SqlalchemyDbTool(DatabaseInfoItem(database_uri=database_uri))


def run_db_query(db_tool: SqlalchemyDbTool, query_callback: Callable[[Session, ...], Any], **kwargs) -> Any:
    if 'session' in kwargs:
        return query_callback(**kwargs)
    with db_tool.get_session() as session:
        kwargs['session'] = session
        return query_callback(**kwargs)


class DatabaseCopyTool:
    """数据库数据拷贝工具"""

    def __init__(self, source_database_uri: str, target_database_uri: str):
        self.source_db_tool = create_db_tool(source_database_uri)
        self.target_db_tool = create_db_tool(target_database_uri)

    def query_source_db_data(self, query_callback: Callable[[Session], Any], **kwargs) -> Any:
        """查询源数据库数据"""
        return run_db_query(db_tool=self.source_db_tool, query_callback=query_callback, **kwargs)

    def query_target_db_data(self, query_callback: Callable[[Session], Any], **kwargs) -> Any:
        """查询目标数据库数据"""
        return run_db_query(db_tool=self.target_db_tool, query_callback=query_callback, **kwargs)

    def query_source_db_model_list(self, model_class: Type, *criterion) -> list[Any]:
        """查询源数据库的模型列表"""

        def fun_wrap(session: Session) -> list[Any]:
            if criterion:
                return session.query(model_class).filter(*criterion).all()
            return session.query(model_class).all()

        return self.query_source_db_data(query_callback=fun_wrap)

    def query_target_db_model_list(self, model_class: Type, *criterion) -> list[Any]:
        """查询目标数据库的模型列表"""

        def fun_wrap(session: Session) -> list[Any]:
            if criterion:
                return session.query(model_class).filter(*criterion).all()
            return session.query(model_class).all()

        return self.query_target_db_data(query_callback=fun_wrap)

    def save_target_data(self, model, *criterion, exist_update: bool = True, is_commit: bool = True):
        """保存目标数据库数据"""

        def fun_wrap(session: Session):
            if criterion:
                db_query = session.query(type(model)).where(*criterion)
            else:
                db_query = session.query(type(model)).where(type(model).id.__eq__(model.id))
            old_data = db_query.first()
            if old_data:
                if exist_update:
                    db_query.update(sqlalchemy_model_utils.to_dict(model))
                    if is_commit:
                        session.commit()
                    return db_query.first()
                else:
                    return old_data
            else:
                session.add(model)
                session.flush()
                if is_commit:
                    session.commit()
                return model

        return self.query_target_db_data(query_callback=fun_wrap)


if __name__ == '__main__':
    db_copy_tool = DatabaseCopyTool(source_database_uri='postgresql://postgres:123456@127.0.0.1:5432/test1-db',
                                    target_database_uri='postgresql://postgres:123456@127.0.0.1:5432/test2-db')

    print('*' * 20, '复制分组信息', '*' * 20)
    # old_po.GroupInfoPo和 new_po.GroupInfoPo都应该是Model类，不需要单引号括起来，这里是因为没有这些Model类，做个测试示例
    old_group_list = db_copy_tool.query_source_db_model_list('old_po.GroupInfoPo')
    for old_group in old_group_list:
        new_group = sqlalchemy_model_utils.copy_model_data(old_group, 'new_po.GroupInfoPo')
        db_copy_tool.save_target_data(new_group)



from typing import Any

from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService


class LinkPoService(PoService):
    """
    使用示例：link_po_service = LinkPoService(app_info.db_code, LinkInfoPo)
    """
    _table_name_ = "tb_link_info"

    def query_by_type_code(self, type_code: str) -> list[Any]:
        return self.query_more(self.model_type.type_code == type_code)



"""
pickle序列化与反序列化工具
"""
import pickle


def save_to_file(obj_list: list[object], save_file: str):
    """保存到文件"""
    with open(save_file, "wb") as f:
        # 序列化
        pickle.dump(len(obj_list), f)
        for tmp_obj in obj_list:
            # 序列化
            pickle.dump(tmp_obj, f)


def parse_to_obj(save_file: str) -> list:
    """保存到文件"""
    with open(save_file, "rb") as f:
        list_size = pickle.load(f)
        # 反序列
        return [pickle.load(f) for tmp in range(list_size)]



"""
request请求工具
"""
import socket
import re
from typing import Mapping


def is_mobile(user_agent: str) -> bool:
    """判断是否是手机"""
    if user_agent is None:
        return False
    user_agent = user_agent.lower()
    mobile_agent_list = ['ipad', 'iphone os', 'midp', 'rv:1.2.3.4', 'ucweb', 'android', 'windows ce',
                         'windows mobile', 'webview', 'mobile', 'iphone']
    for tmp_mobile_agent in mobile_agent_list:
        if re.search(tmp_mobile_agent, user_agent):
            return True
    return False


def is_json(headers: dict | Mapping) -> bool:
    """是否是json请求"""
    if isinstance(headers, dict) or isinstance(headers, Mapping):
        return headers.get('X-Requested-With') == 'XMLHttpRequest' or (
                headers.get('Accept') and 'application/json' in headers.get('Accept').lower())


def is_valid_ipv6_address(ip_address):
    """是否是合法ip6地址"""
    try:
        socket.inet_pton(socket.AF_INET6, ip_address)
        return True
    except socket.error:
        return False


def is_valid_ipv4_address(ip_address):
    """是否是合法ip4地址"""
    try:
        socket.inet_aton(ip_address)
        return True
    except socket.error:
        return False




"""
calmjs 工具
- 压缩：pip install calmjs.parse -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
from calmjs.parse import es5
from calmjs.parse.unparsers.es5 import minify_print, pretty_print, pretty_printer, minify_printer, Unparser
from calmjs.parse.rules import indent
from calmjs.parse.rules import obfuscate

import json
from io import StringIO
from calmjs.parse.sourcemap import encode_sourcemap, write


def js_minify(js_code: str, obfuscate: bool = False, obfuscate_globals: bool = False):
    """压缩js"""
    program = es5(js_code, with_comments=True)
    return minify_print(program, obfuscate=obfuscate, obfuscate_globals=obfuscate_globals)


def js_pretty(js_code: str, obfuscate: bool = False, obfuscate_globals: bool = False):
    """漂亮打印js"""
    program = es5(js_code, with_comments=True)
    return pretty_print(program, indent_str='    ')


def js_sourcemap(js_code: str, source_js: str, min_js_file: str) -> str:
    """
    js压缩与原js的sourcemap
    :param js_code:
    :param source_js: 源js，如：'demo.js'
    :param min_js_file: 压缩js文件，如：'demo.min.js'
    :return: sourcemap的json字符串
    """
    program = es5(js_code, with_comments=True)
    program.sourcepath = source_js
    stream_m = StringIO()
    print_m = minify_printer(obfuscate=True, obfuscate_globals=True)
    sourcemap_m = encode_sourcemap(min_js_file, *write(print_m(program), stream_m))
    return json.dumps(sourcemap_m, indent=2, sort_keys=True)


def js_obfuscate(js_code: str):
    """
    混淆js代码
    :param js_code:
    :return: 混淆后的代码
    """
    pretty_obfuscate = Unparser(rules=(obfuscate(obfuscate_globals=False), indent(indent_str='    '),))
    math_module = es5(js_code)
    return ''.join(c.text for c in pretty_obfuscate(math_module))





from typing import Any

from afeng_tools.fastapi_tool.common.po_service.category_po_service_ import CategoryPoService
from afeng_tools.fastapi_tool.common.service.base_service import BaseService


class CategoryService(BaseService):
    """
    使用示例：category_service = CategoryService(app_info.db_code, CategoryInfoPo)
    """

    po_service_type = CategoryPoService

    def _up_tree_category(self, category_code: int, category_dict: dict[int, Any]) -> list[Any]:
        """向上获取递归获取分类信息"""
        category_list = []
        tmp_category = category_dict.get(category_code)
        if tmp_category:
            if tmp_category.parent_code:
                category_list.extend(self._up_tree_category(tmp_category.parent_code, category_dict))
            category_list.append(tmp_category)
        return category_list

    def _down_tree_category(self, category_code: int, category_dict: dict[int, Any],
                            recursion: bool = False) -> list[Any]:
        """
        向下获取递归获取分类信息， 第一项是当前分类
        :param category_code:
        :param category_dict:
        :param recursion: 是否层级递归
        :return:
        """
        category_list = []
        tmp_category = category_dict.get(category_code)
        if tmp_category:
            category_list.append(tmp_category)
        for tmp_po in category_dict.values():
            if tmp_po.parent_code == category_code:
                category_list.append(tmp_po)
                if recursion:
                    category_list.extend(self._down_tree_category(tmp_po.code, category_dict, recursion=recursion))
        return category_list

    def get_category_list(self, category_code: int, group_code: int = None, up: bool = True) -> list[Any]:
        """
        通过分类编码获取分类列表: 如：['程序开发','Java','Java虚拟机']
        :param group_code: 分组编码
        :param category_code:
        :param up:
            当为True时，向上获取，如当前是Java虚拟机,获取到的是['程序开发','Java','Java虚拟机']
            当为False时，向下获取，如当前是Java,获取到的是['Java基础','Java安全','Java虚拟机', ,'JavaWeb开发']
        :return: 分类列表
        """
        if group_code is None:
            category_info = self.po_service.get_one(self.po_model_type.code == category_code)
            group_code = category_info.group_code
        category_dict = {tmp.code: tmp for tmp in self.po_service.query_group_data(group_code)}
        return self._up_tree_category(category_code, category_dict) if up else self._down_tree_category(category_code,
                                                                                                        category_dict)

    def get_up_category_title_list(self, category_code: int, group_code: int = None) -> list[str]:
        """获取向上的分类标题"""
        category_list = self.get_category_list(group_code=group_code, category_code=category_code, up=True)
        return [tmp.title for tmp in category_list]



from util.flask.flask_models import UrlPatternItem
from web import index, user, audio, download

url_pattern_list: list[UrlPatternItem] = [
    UrlPatternItem(url_prefix=f'/', blueprint=index.app),
    UrlPatternItem(url_prefix=f'/{user.app_name}', blueprint=user.app),
    UrlPatternItem(url_prefix=f'/{download.app_name}', blueprint=download.app),
    UrlPatternItem(url_prefix=f'/{audio.app_name}', blueprint=audio.app),
]




"""
下载工具
    安装：pip install certifi -i https://pypi.tuna.tsinghua.edu.cn/simple/  -U
         pip install requests -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
         pip install tqdm -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import os
from typing import Optional

import certifi
import requests
from tqdm.asyncio import tqdm

from afeng_tools.http_tool.decorator.http_auto_retry_tools import auto_retry
from afeng_tools.log_tool.loguru_tools import get_logger, log_error

logger = get_logger()

DEFAULT_USER_AGENT = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36 Edg/107.0.1418.24'


@auto_retry(max_retries=3)
def download_chunk_file(download_url: str, save_path: str, save_file_name=None, chunk_size: int = 1024,
                        params: Optional[dict] = None, headers: Optional[dict] = None) -> str:
    """
    分块断点续传下载
    :param download_url: 下载url
    :param save_path: 本地保存路径
    :param save_file_name: 保存文件名
    :param chunk_size: 每块大小
    :param params: 请求参数
    :param headers: 请求头
    :return: 本地保存的文件路径
    """
    if headers is None:
        headers = {
            "User-Agent": DEFAULT_USER_AGENT
        }
    if 'User-Agent' not in headers:
        headers['User-Agent'] = DEFAULT_USER_AGENT
    verify = False
    if download_url.startswith('https'):
        verify = certifi.where()
    response = requests.get(download_url, params=params, headers=headers, stream=True, verify=verify, timeout=(10, 20))
    # 记录下载文件的大小
    total_length = int(response.headers.get('content-length', 0))
    if total_length == 0:
        return download_file(download_url=download_url, save_path=save_path, save_file_name=save_file_name,
                             params=params, headers=headers)
    # 保存文件名
    if not save_file_name:
        save_file_name = download_url.rsplit('/', maxsplit=1)[-1].split('?', maxsplit=1)[0]
    os.makedirs(save_path, exist_ok=True)
    # 保存文件路径
    save_file = os.path.join(save_path, save_file_name)
    # 临时下载文件
    down_tmp_file = save_file + '.down-tmp'
    # 本地已经下载的文件大小
    down_size = 0
    if os.path.exists(save_file):
        down_size = os.path.getsize(save_file)
    elif os.path.exists(down_tmp_file):
        down_size = os.path.getsize(down_tmp_file)
    if down_size >= total_length:
        logger.info(f'The file has been downloaded to the local, down_url[{download_url}]')
        return save_file
    else:
        headers['Range'] = f'bytes={down_size}-{total_length}'
        logger.info(f'[begin-download][{total_length}-{down_size}]:[{download_url}]')
        response = requests.get(download_url, params=params, headers=headers, stream=True, verify=verify,
                                timeout=(10, 20))

        # 记录下载文件的大小
        with open(down_tmp_file, 'ab') as fd, tqdm(initial=down_size, desc="正在下载", total=total_length,
                                                   unit='B', unit_scale=True, unit_divisor=1024) as bar:
            # 指针移到末尾
            fd.seek(down_size)
            for chunk in response.iter_content(chunk_size=chunk_size):
                if chunk:
                    size = fd.write(chunk)
                    fd.flush()
                    bar.update(size)
        os.rename(down_tmp_file, save_file)
        return save_file


@auto_retry(max_retries=3)
def download_file(download_url, save_path: str, save_file_name=None, buffer_size: int = 1024,
                  params: Optional[dict] = None, headers: Optional[dict] = None):
    """
    下载文件(不支持断点续传，只能一次性下载完)
    :param download_url: 文件url
    :param save_path: 文件本地保存路径
    :param save_file_name: 保存文件名
    :param buffer_size: 流读取，每次读取大小
    :param params: 请求参数
    :param headers: 请求头
    :return: 下载成功后，返回本地保存文件路径;下载失败后，返回None
    """
    if headers is None:
        headers = {
            "User-Agent": DEFAULT_USER_AGENT
        }
    if 'User-Agent' not in headers:
        headers[
            'User-Agent'] = DEFAULT_USER_AGENT
    verify = False
    if download_url.startswith('https'):
        verify = certifi.where()
    # 请求结果
    response = requests.get(download_url, params=params, headers=headers, stream=True, verify=verify, timeout=(10, 20))
    if response.status_code == 200:
        # 记录下载文件的大小
        total_length = int(response.headers.get('content-length', 0))
        # 保存文件名
        if not save_file_name:
            save_file_name = download_url.rsplit('/', maxsplit=1)[-1].split('?', maxsplit=1)[0]
        os.makedirs(save_path, exist_ok=True)
        # 保存文件路径
        save_file = os.path.join(save_path, save_file_name)
        # 临时下载文件
        down_tmp_file = save_file + '.down-tmp'
        if os.path.exists(save_file):
            logger.info(f'The file has been downloaded to the local, down_url[{download_url}]')
        else:
            # 记录下载文件的大小
            with open(down_tmp_file, 'ab') as fd, tqdm(initial=0, desc="正在下载", total=total_length,
                                                       unit='B', unit_scale=True, unit_divisor=1024) as bar:
                for chunk in response.iter_content(chunk_size=buffer_size):
                    if chunk:
                        size = fd.write(chunk)
                        fd.flush()
                        bar.update(size)
            os.rename(down_tmp_file, save_file)
            return save_file
    else:
        response.encoding = 'utf-8'
        log_error(logger, f'文件下载失败！[{download_url}]：错误信息[{response.status_code}]{response.text}')

# TODO 多线程下载



"""
百度网盘枚举
"""
from enum import Enum


class BaidupanConfigKeyEnum(Enum):
    app_name = 'app_name'
    app_id = 'app_id'
    app_key = 'app_key'
    secret_key = 'secret_key'
    sign_key = 'sign_key'
    redirect_url = 'redirect_url'
    auth_qrcode_image = 'auth_qrcode_image'
    token_file = 'token_file'
    pan_root_path = 'pan_root_path'




import json
import os.path
from typing import Any, Optional, Mapping, Sequence

from pydantic import BaseModel
from starlette.background import BackgroundTask
from starlette.requests import Request
from starlette.responses import FileResponse, Response, RedirectResponse

from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.fastapi_tool.common.service import fastapi_error_service
from afeng_tools.fastapi_tool.core.fastapi_response import json_resp
from afeng_tools.fastapi_tool.fastapi_jinja2_tools import create_template_response
from afeng_tools.sqlalchemy_tools.core.sqlalchemy_base_model import is_model_instance
from afeng_tools.sqlalchemy_tools.tool import sqlalchemy_model_tools


def resp_404(message: str | Sequence = '页面没找到！', request: Request = None, context_data: dict = None,
             app_info: AppInfo = None):
    return fastapi_error_service.handle_404(message=message, request=request, context_data=context_data,
                                            app_info=app_info)


def resp_501(message: str | Sequence, request: Request = None, context_data: dict = None, app_info: AppInfo = None):
    return fastapi_error_service.handle_501(message=message, request=request, context_data=context_data,
                                            app_info=app_info)


def resp_500(message: str | Sequence = '服务器内部错误！', request: Request = None, context_data: dict = None,
             app_info: AppInfo = None):
    return fastapi_error_service.handle_500(message=message, request=request, context_data=context_data,
                                            app_info=app_info)


def resp_template(request: Request, template_file: str, context_data: dict[str, Any]):
    """响应模板"""
    return create_template_response(request=request, template_file=template_file, context=context_data)


def resp_json(data: Any = None, error_no: int = 0, message: str | Sequence = 'success', app_info: AppInfo = None):
    if is_model_instance(data) or (data and isinstance(data, list) and len(data) > 0 and is_model_instance(data[0])):
        data = json.loads(sqlalchemy_model_tools.to_json(data))
    if isinstance(data, BaseModel):
        data = data.model_dump(mode='json')
    return json_resp(error_no=error_no, message=message, data=data)


def resp_file(file_path: str, file_name: str = None, download_flag: bool = False,
              context_data: dict = None, app_info: AppInfo = None) -> Response:
    """响应文件"""
    if not os.path.exists(file_path):
        return resp_404(message='您访问的资源不存在！', context_data=context_data, app_info=app_info)
    response = FileResponse(file_path)
    with open(file_path, "rb") as file:
        if download_flag:
            if file_name is None:
                file_name = os.path.split(file_path)[1]
            response.headers["Content-Disposition"] = f"attachment; filename={file_name}"
        response.body = file.read()
        return response


def redirect(target_url: str, status_code: int = 307,
             headers: Optional[Mapping[str, str]] = None,
             background: Optional[BackgroundTask] = None, app_info: AppInfo = None) -> RedirectResponse:
    """重定向"""
    return RedirectResponse(target_url, status_code=status_code, headers=headers, background=background)



"""
request请求工具
"""
import re
from typing import Mapping

from afeng_tools.http_tool import http_request_tools


def is_mobile(user_agent: str) -> bool:
    """判断是否是手机"""
    return http_request_tools.is_mobile(user_agent)


def is_json(headers: dict | Mapping) -> bool:
    """是否是json请求"""
    return http_request_tools.is_json(headers)




"""
    xpan

    xpanapi  # noqa: E501

    The version of the OpenAPI document: 0.1
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from openapi_client.api_client import ApiClient, Endpoint as _Endpoint
from openapi_client.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)


class MultimediafileApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.xpanfilelistall_endpoint = _Endpoint(
            settings={
                'response_type': (dict,),
                'auth': [],
                'endpoint_path': '/rest/2.0/xpan/multimedia?method=listall&openapi=xpansdk',
                'operation_id': 'xpanfilelistall',
                'http_method': 'GET',
                'servers': [
                    {
                        'url': "https://pan.baidu.com",
                        'description': "No description provided",
                    },
                ]
            },
            params_map={
                'all': [
                    'access_token',
                    'path',
                    'recursion',
                    'web',
                    'start',
                    'limit',
                    'order',
                    'desc',
                ],
                'required': [
                    'access_token',
                    'path',
                    'recursion',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'access_token':
                        (str,),
                    'path':
                        (str,),
                    'recursion':
                        (int,),
                    'web':
                        (str,),
                    'start':
                        (int,),
                    'limit':
                        (int,),
                    'order':
                        (str,),
                    'desc':
                        (int,),
                },
                'attribute_map': {
                    'access_token': 'access_token',
                    'path': 'path',
                    'recursion': 'recursion',
                    'web': 'web',
                    'start': 'start',
                    'limit': 'limit',
                    'order': 'order',
                    'desc': 'desc',
                },
                'location_map': {
                    'access_token': 'query',
                    'path': 'query',
                    'recursion': 'query',
                    'web': 'query',
                    'start': 'query',
                    'limit': 'query',
                    'order': 'query',
                    'desc': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json; charset=UTF-8'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.xpanmultimediafilemetas_endpoint = _Endpoint(
            settings={
                'response_type': (dict,),
                'auth': [],
                'endpoint_path': '/rest/2.0/xpan/multimedia?method=filemetas&openapi=xpansdk',
                'operation_id': 'xpanmultimediafilemetas',
                'http_method': 'GET',
                'servers': [
                    {
                        'url': "https://pan.baidu.com",
                        'description': "No description provided",
                    },
                ]
            },
            params_map={
                'all': [
                    'access_token',
                    'fsids',
                    'thumb',
                    'extra',
                    'dlink',
                    'path',
                    'needmedia',
                ],
                'required': [
                    'access_token',
                    'fsids',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'access_token':
                        (str,),
                    'fsids':
                        (str,),
                    'thumb':
                        (str,),
                    'extra':
                        (str,),
                    'dlink':
                        (str,),
                    'path':
                        (str,),
                    'needmedia':
                        (int,),
                },
                'attribute_map': {
                    'access_token': 'access_token',
                    'fsids': 'fsids',
                    'thumb': 'thumb',
                    'extra': 'extra',
                    'dlink': 'dlink',
                    'path': 'path',
                    'needmedia': 'needmedia',
                },
                'location_map': {
                    'access_token': 'query',
                    'fsids': 'query',
                    'thumb': 'query',
                    'extra': 'query',
                    'dlink': 'query',
                    'path': 'query',
                    'needmedia': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json; UTF-8'
                ],
                'content_type': [],
            },
            api_client=api_client
        )

    def xpanfilelistall(
        self,
        access_token,
        path,
        recursion,
        **kwargs
    ):
        """xpanfilelistall  # noqa: E501

        listall  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.xpanfilelistall(access_token, path, recursion, async_req=True)
        >>> result = thread.get()

        Args:
            access_token (str):
            path (str):
            recursion (int):

        Keyword Args:
            web (str): [optional]
            start (int): [optional]
            limit (int): [optional]
            order (str): [optional]
            desc (int): [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            str
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['access_token'] = \
            access_token
        kwargs['path'] = \
            path
        kwargs['recursion'] = \
            recursion
        return self.xpanfilelistall_endpoint.call_with_http_info(**kwargs)

    def xpanmultimediafilemetas(
        self,
        access_token,
        fsids,
        **kwargs
    ):
        """xpanmultimediafilemetas  # noqa: E501

        multimedia filemetas  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.xpanmultimediafilemetas(access_token, fsids, async_req=True)
        >>> result = thread.get()

        Args:
            access_token (str):
            fsids (str):

        Keyword Args:
            thumb (str): [optional]
            extra (str): [optional]
            dlink (str): 下载地址。下载接口需要用到dlink。. [optional]
            path (str): 查询共享目录或专属空间内文件时需要。共享目录格式： /uk-fsid（其中uk为共享目录创建者id， fsid对应共享目录的fsid）。专属空间格式：/_pcs_.appdata/xpan/。. [optional]
            needmedia (int): [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            str
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['access_token'] = \
            access_token
        kwargs['fsids'] = \
            fsids
        return self.xpanmultimediafilemetas_endpoint.call_with_http_info(**kwargs)




import enum
import json
from typing import Type, Any, Optional, Iterable

from fastapi import Form
from fastapi.encoders import jsonable_encoder
from pydantic import BaseModel
from sqlalchemy import String, func, Text, Boolean, ARRAY
from starlette.requests import Request
from starlette.responses import JSONResponse, HTMLResponse

from afeng_tools.fastapi_tool.fastapi_response_tools import resp_json
from afeng_tools.id_code_tool import id_code_tools
from afeng_tools.layui_tool import layui_template_tools
from afeng_tools.layui_tool.core.layui_models import LayUiFormItem, LayUiTableCol, LayUiTableActionItem, LayUiTableData
from afeng_tools.pydantic_tool.model.common_models import EnumItem
from afeng_tools.sqlalchemy_tools.crdu import base_crdu
from afeng_tools.sqlalchemy_tools.tool import orm_model_tools


class AdminViewsTemplate:
    def __init__(self, module_code: str, orm_model_class: Type, search_form_item_list: list[LayUiFormItem],
                 db_code='default'):
        """
        Admin管理Views模板
        :param module_code: 模块编码
        :param orm_model_class: orm模型类
        :param search_form_item_list: 用于查询的表单项列表
        """
        self.module_code = module_code
        self.orm_model_class = orm_model_class
        self.search_form_item_list = search_form_item_list
        self.db_code = db_code

    def _create_form_item_list(self, old_data=None):
        form_item_list = []
        for c in self.orm_model_class.__table__.columns:

            if issubclass(c.type.python_type, enum.Enum):
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name, type='select',
                                              select_list=[((tmp.value if isinstance(tmp.value, str) else tmp.name),
                                                            tmp.name, False, False) for tmp in
                                                           c.type.python_type])
            elif isinstance(c.type, Boolean):
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name, type='switch',
                                              switch_item=('1', '开启|关闭', False, False))
            elif isinstance(c.type, ARRAY):
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name, type='array')
            elif issubclass(c.type.python_type, int):
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name, type='number')
            elif isinstance(c.type, Text):
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name, type='textarea')
            else:
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name)
            if old_data is not None:
                if issubclass(c.type.python_type, enum.Enum):
                    tmp_select_list = []
                    for tmp_value, tmp_title, tmp_selected, tmp_disabled in tmp_form_item.select_list:
                        old_data_field_value = getattr(old_data, c.name)
                        if isinstance(old_data_field_value, enum.Enum):
                            if isinstance(old_data_field_value.value, EnumItem):
                                old_data_field_value = old_data_field_value.value.value
                            else:
                                old_data_field_value = old_data_field_value.value
                        if tmp_value == old_data_field_value:
                            tmp_selected = True
                        tmp_select_list.append((tmp_value, tmp_title, tmp_selected, tmp_disabled))
                    tmp_form_item.select_list = tmp_select_list
                elif isinstance(c.type, Boolean):
                    tmp_value, tmp_title, tmp_checked, tmp_disabled = tmp_form_item.switch_item
                    if getattr(old_data, c.name):
                        tmp_checked = True
                    tmp_form_item.switch_item = (tmp_value, tmp_title, tmp_checked, tmp_disabled)
                elif isinstance(c.type, ARRAY):
                    tmp_value = getattr(old_data, c.name)
                    if tmp_value and isinstance(tmp_value, Iterable):
                        tmp_form_item.default = ','.join([str(tmp) for tmp in tmp_value])
                else:
                    print(type(old_data), c.name)
                    tmp_value = getattr(old_data, c.name)
                    if tmp_value is not None:
                        print(type(tmp_value), c.name)
                        tmp_form_item.default = getattr(old_data, c.name)
            form_item_list.append(tmp_form_item)
        return form_item_list

    async def index_page(self) -> HTMLResponse:
        """列表页面"""
        model_title = self.orm_model_class.__doc__
        col_list = [
            LayUiTableCol(field=c.name, title=c.comment, sort=True, is_enum=issubclass(c.type.python_type, enum.Enum))
            for c in
            self.orm_model_class.__table__.columns]
        return HTMLResponse(layui_template_tools.render_table_template(title=f'{model_title}-管理',
                                                                       search_form_item_list=self.search_form_item_list,
                                                                       data_url=f'/admin/api/{self.module_code}/list',
                                                                       col_list=col_list,
                                                                       add=True,
                                                                       add_html_url=f'/admin/{self.module_code}/add',
                                                                       edit=True,
                                                                       edit_html_url=f'/admin/{self.module_code}/update',
                                                                       delete=True,
                                                                       delete_url=f'/admin/api/{self.module_code}/delete',
                                                                       more_delete=True,
                                                                       more_delete_url=f'/admin/api/{self.module_code}/more_delete',
                                                                       action_list=[
                                                                           LayUiTableActionItem(title='展示',
                                                                                                url=f'/admin/{self.module_code}/show')
                                                                       ]
                                                                       ))

    async def add_page(self) -> HTMLResponse:
        """添加页面"""
        model_title = self.orm_model_class.__doc__
        return HTMLResponse(layui_template_tools.render_add_template(title=f'添加-{model_title}',
                                                                     form_item_list=self._create_form_item_list(),
                                                                     add_url=f'/admin/api/{self.module_code}/add'))

    async def update_page(self, request: Request, id_value: int) -> HTMLResponse:
        """修改页面"""
        model_title = self.orm_model_class.__doc__
        old_data = base_crdu.query_one(self.orm_model_class,
                                       self.orm_model_class.id == id_value,
                                       db_code=self.db_code)
        return HTMLResponse(layui_template_tools.render_edit_template(title=f'修改-{model_title}',
                                                                      form_item_list=self._create_form_item_list(
                                                                          old_data),
                                                                      edit_url=f'/admin/api/{self.module_code}/update'))

    async def show_page(self, request: Request, id_value: int) -> HTMLResponse:
        """展示页面"""
        model_title = self.orm_model_class.__doc__
        old_data = base_crdu.query_one(self.orm_model_class,
                                       self.orm_model_class.id == id_value,
                                       db_code=self.db_code)
        if old_data:
            form_item_list = [
                LayUiFormItem(title=c.comment, name=c.name, default=getattr(old_data, c.name), readonly=True)
                for c in self.orm_model_class.__table__.columns]
        else:
            form_item_list = [LayUiFormItem(title=c.comment, name=c.name)
                              for c in self.orm_model_class.__table__.columns]
        return HTMLResponse(layui_template_tools.render_show_template(title=f'展示-{model_title}',
                                                                      form_item_list=form_item_list))

    async def add_data(self, request: Request, form_data: BaseModel) -> JSONResponse:
        """添加数据"""
        po = orm_model_tools.py_model_to_orm_model(form_data, self.orm_model_class)
        po = base_crdu.add(po, db_code=self.db_code)
        self.add_data_after_handle(po)
        return resp_json(po)

    async def update_data(self, request: Request, form_data: BaseModel) -> JSONResponse:
        """更新数据"""
        po = orm_model_tools.py_model_to_orm_model(form_data, self.orm_model_class)
        po = base_crdu.update(po, db_code=self.db_code)
        self.update_data_after_handle(po)
        return resp_json(po)

    async def delete_data(self, request: Request, id_value: int) -> JSONResponse:
        """删除数据"""
        old_po = base_crdu.query_one(self.orm_model_class, self.orm_model_class.id == id_value,
                                     db_code=self.db_code)
        base_crdu.delete_by_ids(self.orm_model_class, [id_value], db_code=self.db_code)
        self.delete_data_after_handle(old_po)
        return resp_json('success')

    async def more_delete_data(self, request: Request, ids: str = Form()):
        id_list = json.loads(ids)
        old_po_list = base_crdu.query_all(self.orm_model_class, self.orm_model_class.id.in_(id_list),
                                          db_code=self.db_code)
        base_crdu.delete_by_ids(self.orm_model_class, id_list, db_code=self.db_code)
        for old_po in old_po_list:
            self.delete_data_after_handle(old_po)
        return resp_json('success')

    async def get_data(self, request: Request, id_value: int) -> JSONResponse:
        """查询单条数据"""
        return resp_json(base_crdu.query_one(self.orm_model_class, self.orm_model_class.id == id_value,
                                             db_code=self.db_code))

    async def list_data(self, request: Request, page: Optional[int] = 1, limit: Optional[int] = 15,
                        searchParams: Optional[str] = None):
        """列出所有数据"""
        search_params = None
        if searchParams:
            search_params = json.loads(searchParams)
        query = base_crdu.create_query(self.orm_model_class, db_code=self.db_code)
        if search_params:
            search_field_list = [tmp.name for tmp in self.search_form_item_list]
            for tmp_key, tmp_value in search_params.items():
                if tmp_key in search_field_list and tmp_value is not None and len(tmp_value) > 0:
                    query = query.filter(
                        func.cast(getattr(self.orm_model_class, tmp_key), String).ilike(f'%{tmp_value}%'))
        query = query.order_by(self.orm_model_class.add_time.desc())
        return jsonable_encoder(LayUiTableData(
            total=query.count(),
            data_list=query.offset((page - 1) * limit).limit(limit).all()
        ).model_dump())

    def add_data_after_handle(self, po_model: Any):
        """添加数据后处理"""
        code_column = [c for c in self.orm_model_class.__table__.columns if c.name == 'code']
        if code_column:
            setattr(po_model, 'code', id_code_tools.get_code_by_id(po_model.id))
            base_crdu.update(po_model, db_code=self.db_code)

    def update_data_after_handle(self, po_model: Any):
        """更新数据后处理"""
        pass

    def delete_data_after_handle(self, po_model: Any):
        """删除数据后处理"""
        pass



"""
百度网盘授权工具
"""
import webbrowser
from datetime import datetime
from typing import Literal

import openapi_client
from afeng_tools.baidu_pan_tool.core.baidu_pan_decorator_tools import auto_auth_api
from afeng_tools.baidu_pan_tool.core.baidu_pan_models import TokenInfo, DeviceCodeInfo
from afeng_tools.http_tool import http_params_tools
from afeng_tools.log_tool.loguru_tools import get_logger, log_error
from afeng_tools.math_tool.random_tools import random_state
from openapi_client.api.auth_api import AuthApi

logger = get_logger()


def get_authorization_code(app_id: str, app_key: str,
                           redirect_uri: str = 'oob',
                           display: Literal['page', 'popup', 'dialog', 'mobile', 'tv', 'pad'] | str = 'page',
                           qrcode: int = 1, force_login: int = 0) -> str:
    """
    获取授权码Code
    :param app_id: AppID
    :param app_key: 应用的AppKey。
    :param redirect_uri: 授权后要回调的地址URL
            可传 redirect_uri=oob,回调后会返回一个平台提供默认回调地址：http://openapi.baidu.com/oauth/2.0/login_success。
    :param display:
            page: 全屏形式的授权页面(默认)，适用于 web 应用。
            popup: 弹框形式的授权页面，适用于桌面软件应用和 web 应用
            dialog: 浮层形式的授权页面，只能用于站内 web 应用
            mobile: Iphone/Android 等智能移动终端上用的授权页面，适用于 Iphone/Android 等智能移动终端上的应用
            tv: 电视等超大显示屏使用的授权页面
            pad: 适配 IPad/Android 等智能平板电脑使用的授权页面
    :param qrcode: 	让用户通过扫二维码的方式登录百度账号时，可传递 “qrcode=1”
    :param force_login: 当需要加载登录页时强制用户用户输入用户名和密码，不会从 Cookies 中读取百度用户的登录状态时，传递 “force_login=1”
    :return: 获取到的授权码 code 有效期 10 分钟，且仅一次有效。
    """
    params = {
        'response_type': 'code',
        'client_id': app_key,
        'redirect_uri': redirect_uri,
        'scope': 'basic,netdisk',
        'device_id': app_id,
        'state': random_state(),
        'display': display,
        'qrcode': qrcode,
        'force_login': force_login
    }
    auth_url = 'http://openapi.baidu.com/oauth/2.0/authorize?' + http_params_tools.url_encode_params(params)
    logger.info(f'[BaiduPan]登录认证url：{auth_url}')
    # 浏览器打开授权， 然后在下面输入授权码
    webbrowser.open(auth_url)
    # 2、从百度授权后重定向的链接中获取到code值
    auth_code = input('请输入授权码：')
    return auth_code


@auto_auth_api
def get_token_authorization_code(auth_code: str, app_key: str, secret_key: str,
                                 redirect_uri: str = 'oob', api_instance: AuthApi = None) -> TokenInfo:
    """
    授权码模式: 用户授权后生成授权码 Code，开发者应用通过 Code 换取 Access Token。
    :param auth_code: 授权码Code
    :param app_key: 应用的AppKey
    :param secret_key: 应用的SecretKey
    :param redirect_uri: 应用设置的授权回调地址
    :param api_instance: 自动注入AuthApi
    :return: Access Token 有效期30天，过期后支持刷新。
    """
    try:
        api_response = api_instance.oauth_token_code2token(auth_code, app_key, secret_key, redirect_uri)
        return TokenInfo(
            access_token=api_response['access_token'],
            expires_in=api_response['expires_in'],
            refresh_token=api_response['refresh_token'],
            scope=api_response['scope'],
            token_time=datetime.now()
        )
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling AuthApi->oauth_token_code2token", e)


@auto_auth_api
def get_device_code(app_key: str, api_instance: AuthApi = None) -> DeviceCodeInfo:
    """
    获取设备码 device_code和user_code
    :param app_key: 应用的AppKey
    :param api_instance: 自动注入的 AuthApi
    :return: Device Code只能使用一次
    """
    scope = "basic,netdisk"
    try:
        api_response = api_instance.oauth_token_device_code(client_id=app_key, scope=scope)
        return DeviceCodeInfo(
            device_code=api_response['device_code'],
            user_code=api_response['user_code'],
            verification_url=api_response['verification_url'],
            qrcode_url=api_response['qrcode_url'],
            expires_in=api_response['expires_in'],
            interval=api_response['interval']
        )
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling AuthApi->oauth_token_device_code", e)


@auto_auth_api
def get_token_device_code(device_code: str, app_key: str, secret_key: str, api_instance: AuthApi = None):
    """
    设备码模式: 获取设备码，用户授权后，开发者应用通过设备码换取 Access Token。
        轮询此接口每两次请求时间间隔应大于5秒。
    :param device_code: 获取设备码
    :param app_key: 应用的AppKey
    :param secret_key: 应用的SecretKey
    :param api_instance: 自动注入的 AuthApi
    :return: Access Token过期后支持刷新。
    """
    try:
        api_response = api_instance.oauth_token_device_token(code=device_code,
                                                             client_id=app_key, client_secret=secret_key)
        return TokenInfo(
            access_token=api_response['access_token'],
            expires_in=api_response['expires_in'],
            refresh_token=api_response['refresh_token'],
            scope=api_response['scope'],
            token_time=datetime.now()
        )
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling AuthApi->oauth_token_device_token", e)


@auto_auth_api
def refresh_token(refresh_token_value: str, app_key: str,
                  secret_key: str, api_instance: AuthApi = None) -> TokenInfo:
    """
    刷新Token：
    :param refresh_token_value: 获取 Access Token时返回的 refresh_token 值， refresh_token 只支持使用一次， 使用后失效
    :param app_key: 应用的AppKey
    :param secret_key: 应用的SecretKey
    :param api_instance: 自动注入的 AuthApi
    :return: 刷新后的 Access Token 有效期仍为 30 天
    """
    try:
        api_response = api_instance.oauth_token_refresh_token(refresh_token_value, app_key, secret_key)
        return TokenInfo(**{
            'access_token': api_response['access_token'],
            'expires_in': api_response['expires_in'],
            'refresh_token': api_response['refresh_token'],
            'scope': api_response['scope'],
            'token_time': datetime.now()
        })
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling AuthApi->oauth_token_refresh_token", e)




from abc import ABCMeta, abstractmethod

from starlette.requests import Request

from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.fastapi_tool.core.fastapi_items import ResponseTemplateResult
from afeng_tools.fastapi_tool.core.fastapi_response import template_resp, json_resp
from afeng_tools.http_tool import http_request_tools


class ErrorService(metaclass=ABCMeta):

    @abstractmethod
    def handle_404(self, message: str = '您访问的资源不存在！', request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        return json_resp(error_no=404, message=message)

    @abstractmethod
    def handle_500(self, message: str = '服务器内部错误！', request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        return json_resp(error_no=500, message=message)

    @abstractmethod
    def handle_501(self, message: str = '操作失败！', request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        return json_resp(error_no=501, message=message)


class DefaultErrorService(ErrorService):
    """默认错误处理"""

    def handle_response(self, request: Request, app_info: AppInfo, error_no: int, title: str, message: str,
                        template_file: str,
                        context_data: dict):
        if request and not http_request_tools.is_json(request.headers) and not (app_info and app_info.is_json_api):
            resp_result = ResponseTemplateResult(title=title, message=message,
                                                 template_file=f'common/{template_file}',
                                                 context_data=context_data)
            app_info = app_info if app_info else request.scope.get('app_info')
            if context_data and 'template_file' in context_data:
                resp_result.template_file = resp_result.context_data['template_file']
            elif app_info:
                resp_result.template_file = f'{app_info.code}/{template_file}'
            return template_resp(request=request, resp_result=resp_result, app_info=app_info)
        return json_resp(error_no=error_no, message=message)

    def handle_404(self, message: str = '您访问的资源不存在！', request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        return self.handle_response(request=request, app_info=app_info, error_no=404, title='404错误页面',
                                    message=message,
                                    template_file='views/error/404.html', context_data=context_data)

    def handle_500(self, message: str = '服务器内部错误！', request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        return self.handle_response(request=request, app_info=app_info, error_no=500, title='500错误页面',
                                    message=message,
                                    template_file='views/error/500.html', context_data=context_data)

    def handle_501(self, message: str = '操作失败！', request: Request = None, context_data: dict = None,
                   app_info: AppInfo = None):
        return self.handle_response(request=request, app_info=app_info, error_no=501, title='操作失败页面',
                                    message=message,
                                    template_file='views/error/501.html', context_data=context_data)



"""
vue工具： pip install vbuild -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
- https://pypi.org/project/vbuild/
"""
import vbuild


def render_vue(vue_file: str | tuple[str]):
    """
    渲染vue
    :param vue_file: mycompo.vue 或   vues/*.vue  或  ["c1.vue", "c2.vue"]
    :return: (html, script, style)
    """

    if isinstance(vue_file, list):
        vue_render = vbuild.render(*vue_file)
    else:
        vue_render = vbuild.render(vue_file)
    return vue_render.html, vue_render.script, vue_render.style


def replace_html_template(html_template_file: str, vue_file: str, vue_template_tag: str = '<!-- HERE -->'):
    """
    替换html模板中的内容
    :param html_template_file: html模板文件
    :param vue_file: mycompo.vue  或   vues/*.vue
    :param vue_template_tag: vue模板标记
    :return: 替换模板内容后的html内容
    """
    with open(html_template_file, encoding='utf-8') as fid:
        template_content = fid.read()
    vue_render = vbuild.render(vue_file)
    return template_content.replace(vue_template_tag, str(vue_render))



"""
分享工具(付费接口)
"""
import json

import requests

from afeng_tools.baidu_pan_tool.core.baidu_pan_models import CreateShareResult, VerifySharePwdResult, QueryShareInfoResult, \
    QueryShareDataListResult, TransferShareResult, TransferShareTaskQueryResult
from afeng_tools.http_tool import http_params_tools
from afeng_tools.log_tool.loguru_tools import log_error, get_logger

logger = get_logger()


def create_share(app_id: str, access_token: str, fs_id_list: list[str], pwd: str,
                 period: int = 7, remark: str = None) -> CreateShareResult:
    """
    创建分享链接
    :param app_id: 	应用ID
    :param access_token: Access Token
    :param fs_id_list: 分享文件id列表，json格式，字符串数组
    :param pwd: 分享密码，长度4位，数字+小写字母组成
    :param period: 分享有效期，单位天
    :param remark: 分享备注
    :return: CreateShareResult
    """
    params = {
        'product': 'netdisk',
        'appid': app_id,
        'access_token': access_token
    }
    form_data = {
        'fsid_list': json.dumps(fs_id_list),
        'period': str(period),
        'pwd': pwd,
        'remark': remark
    }
    headers = {
        'Host': 'pan.baidu.com'
    }
    try:
        post_url = 'https://pan.baidu.com/apaas/1.0/share/set?' + http_params_tools.url_encode_params(params)
        print(post_url)
        response = requests.post(post_url, data=form_data, headers=headers)
        response.encoding = 'utf-8'
        if response.status_code == 200:
            api_response = response.json()
            if api_response['errno'] == 0:
                return CreateShareResult(**api_response)
            else:
                log_error(logger, f"[BaiduPan]创建分享链接[{fs_id_list}]失败:[{api_response}]")
        else:
            log_error(logger, f"[BaiduPan]创建分享链接[{fs_id_list}]失败:[{response.status_code}]{response.text}")
    except Exception as e:
        log_error(logger, f"[BaiduPan]创建分享链接[{fs_id_list}]失败", e)


def verify_share_pwd(app_id: str, access_token: str, short_url: str, pwd: str) -> VerifySharePwdResult:
    """
    分享提取码验证: 本接口用于校验分享链接提取码的正确性。
    :param app_id: 	应用ID
    :param access_token: Access Token
    :param short_url: 分享短链
    :param pwd: 分享密码，长度4位，数字+小写字母组成
    :return: VerifySharePwdResult
    """
    params = {
        'product': 'netdisk',
        'appid': app_id,
        'access_token': access_token,
        'short_url': short_url
    }
    form_data = {
        'pwd': pwd,
    }
    headers = {
        'Host': 'pan.baidu.com'
    }
    try:
        post_url = 'https://pan.baidu.com/apaas/1.0/share/verify?' + http_params_tools.url_encode_params(params)
        response = requests.post(post_url, data=form_data, headers=headers)
        response.encoding = 'utf-8'
        if response.status_code == 200:
            api_response = response.json()
            if api_response['errno'] == 0:
                return VerifySharePwdResult(**api_response)
            else:
                log_error(logger, f"[BaiduPan]验证分享提取码[{short_url}]-[{pwd}]失败:[{api_response}]")
        else:
            log_error(logger,
                      f"[BaiduPan]验证分享提取码[{short_url}]-[{pwd}]失败:[{response.status_code}]{response.text}")
    except Exception as e:
        log_error(logger, f"[BaiduPan]验证分享提取码[{short_url}]-[{pwd}]失败", e)


def query_share_info(app_id: str, access_token: str, short_url: str, s_pwd: str) -> QueryShareInfoResult:
    """
    查询分享详情: 本接口用于查询指定分享链接的链接属性信息，如创建时间、有效期等信息。
    :param app_id: 应用ID
    :param access_token: Access Token
    :param short_url: 分享短链
    :param s_pwd: 加密的提取码
    :return: QueryShareInfoResult
    """
    params = {
        'product': 'netdisk',
        'appid': app_id,
        'access_token': access_token,
        'short_url': short_url
    }
    form_data = {
        'spwd': s_pwd,
    }
    headers = {
        'Host': 'pan.baidu.com'
    }
    try:
        post_url = 'https://pan.baidu.com/apaas/1.0/share/info?' + http_params_tools.url_encode_params(params)
        response = requests.post(post_url, data=form_data, headers=headers)
        response.encoding = 'utf-8'
        if response.status_code == 200:
            api_response = response.json()
            if api_response['errno'] == 0:
                return QueryShareInfoResult(**api_response)
            else:
                log_error(logger, f"[BaiduPan]查询分享详情[{short_url}]-[{s_pwd}]失败:[{api_response}]")
        else:
            log_error(logger,
                      f"[BaiduPan]查询分享详情[{short_url}]-[{s_pwd}]失败:[{response.status_code}]{response.text}")
    except Exception as e:
        log_error(logger, f"[BaiduPan]查询分享详情[{short_url}]-[{s_pwd}]失败", e)


def query_share_data_list(app_id: str, access_token: str, short_url: str, s_pwd: str,
                          dir_path: str = None, page_num: int = 1, page_size: int = 100) -> QueryShareDataListResult:
    """
    查询分享文件信息: 本接口用于获取分享链接内对应的文件信息，包括分享链接的创建用户、创建时间、分享的文件列表和目录结构、分享文件的大小、名称、类型、缩略图、文件ID。
    :param app_id: 应用ID
    :param access_token: Access Token
    :param short_url: 分享短链
    :param s_pwd: 加密的提取码
    :param dir_path: 分享层级目录, 空表示获取外链第一层级文件列表
    :param page_num: 分页号（当dir非空时必传；dir 非根目录时分页参数才有效）
    :param page_size: 每页大小，最大 100（当dir非空时必传）
    :return: QueryShareDataListResult
    """
    params = {
        'product': 'netdisk',
        'appid': app_id,
        'access_token': access_token,
        'short_url': short_url
    }
    form_data = {
        'spwd': s_pwd,
    }
    if dir_path:
        form_data['dir'] = dir_path
        form_data['page'] = page_num,
        form_data['page_size'] = page_size
    headers = {
        'Host': 'pan.baidu.com'
    }
    try:
        post_url = 'https://pan.baidu.com/apaas/1.0/share/info?' + http_params_tools.url_encode_params(params)
        response = requests.post(post_url, data=form_data, headers=headers)
        response.encoding = 'utf-8'
        if response.status_code == 200:
            api_response = response.json()
            if api_response['errno'] == 0:
                return QueryShareDataListResult(**api_response)
            else:
                log_error(logger, f"[BaiduPan]获取分享链接内对应的文件信息[{short_url}]-[{s_pwd}]失败:[{api_response}]")
        else:
            log_error(logger,
                      f"[BaiduPan]获取分享链接内对应的文件信息[{short_url}]-[{s_pwd}]失败:[{response.status_code}]{response.text}")
    except Exception as e:
        log_error(logger, f"[BaiduPan]获取分享链接内对应的文件信息[{short_url}]-[{s_pwd}]失败", e)


def transfer_share(app_id: str, access_token: str, short_url: str, fs_id_list: list[str], to_pan_path: str, s_pwd: str,
                   async_value: int = 2, on_dup: str = 'newcopy') -> TransferShareResult:
    """
    分享文件转存: 本接口用于将分享链接中的内容转存到当前授权用户指定网盘目录下。
    :param app_id: 应用ID
    :param access_token: Access Token
    :param short_url: 分享短链
    :param fs_id_list: 转存文件ID，注意参数为json序列化后的字符串数组，fsid需要带单引号
    :param to_pan_path: 保存路径
    :param s_pwd: 	加密的提取码
    :param async_value: 是否异步，建议全部走异步：2
    :param on_dup: 文件冲突行为，fail-失败，newcopy-复制
    :return: TransferShareResult
    """
    params = {
        'product': 'netdisk',
        'appid': app_id,
        'access_token': access_token,
        'short_url': short_url
    }
    form_data = {
        'spwd': s_pwd,
        'fsid_list': json.dumps(fs_id_list),
        'to_path': to_pan_path,
        'async': async_value,
        'ondup': on_dup
    }
    headers = {
        'Host': 'pan.baidu.com'
    }
    try:
        post_url = 'https://pan.baidu.com/apaas/1.0/share/transfer?' + http_params_tools.url_encode_params(params)
        response = requests.post(post_url, data=form_data, headers=headers)
        response.encoding = 'utf-8'
        if response.status_code == 200:
            api_response = response.json()
            if api_response['errno'] == 0:
                return TransferShareResult(**api_response)
            else:
                log_error(logger, f"[BaiduPan]查询分享详情[{short_url}]-[{s_pwd}]失败:[{api_response}]")
        else:
            log_error(logger,
                      f"[BaiduPan]查询分享详情[{short_url}]-[{s_pwd}]失败:[{response.status_code}]{response.text}")
    except Exception as e:
        log_error(logger, f"[BaiduPan]查询分享详情[{short_url}]-[{s_pwd}]失败", e)


def transfer_share_task_query(app_id: str, access_token: str, task_id:int) -> TransferShareTaskQueryResult:
    """
    转存任务查询: 本接口用于查询转存任务结果
    :param app_id: 	应用ID
    :param access_token: Access Token
    :param task_id: 转存接口返回的task_id
    :return: TransferShareTaskQueryResult
    """
    pass



"""
二维码图片工具类
- pillow: pip install pillow  -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import os.path
import platform
from pathlib import Path

from PIL import Image


def get_blocks():
    """获取二维码用到的方块"""
    if platform.system() == "Windows":
        white_block = '▇'
        black_block = '  '
        new_line = '\n'
    else:
        white_block = '\033[0;37;47m  '
        black_block = '\033[0;37;40m  '
        new_line = '\033[0m\n'
    return white_block, black_block, new_line


def get_box_size(img: Image):
    """获取每一个二维码的每个box像素块的大小（像素）"""
    width, height = img.size
    x1, y1, x2, y2 = 0, 0, 0, 0
    have_black = False
    hava_white = False
    for y in range(height):
        for x in range(width):
            tmp_pix = img.getpixel((x, y))
            if tmp_pix[:3] == (0, 0, 0) and not have_black:
                x1, y1 = x, y
                have_black = True
            if have_black and tmp_pix[:3] == (255, 255, 255):
                x2, y2 = x, y
                hava_white = True
                break
        if hava_white:
            break
    for x in range(x1, x2):
        for y in range(x1, x2):
            tmp_pix = img.getpixel((y, x))
            if tmp_pix[:3] == (255, 255, 255):
                #  每个黑色格子的像素点大小
                cell_width = y - y1
                cell_height = x - x1
                return cell_width, cell_height


def print_qrcode_image(qrcode_image_file):
    """打印二维码图片到控制台"""
    img = Image.open(qrcode_image_file).convert("RGB")
    box_width, box_height = get_box_size(img)
    width, height = img.size
    white_block, black_block, new_line = get_blocks()
    output = ''
    for y in range(0, height, box_height):
        for x in range(0, width, box_width):
            tmp_pix = img.getpixel((x, y))
            if tmp_pix[:3] == (0, 0, 0):
                # 黑色像素
                output += white_block
            elif tmp_pix[:3] == (255, 255, 255):
                # 白色像
                output += black_block
        output += new_line
    print(output)
    return output


def print_qrcode(qrcode_image_file: str):
    im = Image.open(qrcode_image_file)
    width, height = 33, 33
    im = im.resize((width, height), Image.NEAREST)
    text = ''
    for w in range(width):
        for h in range(height):
            res = im.getpixel((h, w))
            text += '  ' if res == 0 else '██'
        text += '\n'
    print(text)
    return text


if __name__ == '__main__':
    qrcode_image = os.path.join(Path(__file__).parent, '../../../tool/image/data/qr_test.png')
    # print_qrcode_image(qrcode_image)
    print_qrcode(qrcode_image)


"""
fastapi的json工具类
"""
import json
from typing import Any
from fastapi.encoders import jsonable_encoder


def to_json_str(obj_data: Any, indent: int = None, allow_nan: bool = False) -> str:
    """转换为json字符串"""
    return json.dumps(
        jsonable_encoder(obj_data),
        ensure_ascii=False,
        allow_nan=allow_nan,
        indent=indent,
        separators=(",", ":"),
    )


def to_json_bytes(obj_data: Any, indent: int = None, allow_nan: bool = False) -> bytes:
    """转换为json字节"""
    return to_json_str(obj_data, allow_nan=allow_nan, indent=indent).encode("utf-8")



from typing import Optional, Any

from pydantic import BaseModel, Field


class EnumItem(BaseModel):
    # 是否是默认项
    is_default: Optional[bool] = False
    # 标题
    title: str = Field(default=None, title='枚举标题')
    value: Optional[str] = Field(default=None, title='枚举值')
    data_dict: Optional[dict[str, Any]] = Field(title='数据字典', default=dict())


class LinkItem(EnumItem):
    """链接项"""
    href: str = Field(title='链接地址')
    title: str = Field(title='链接标题')
    code: Optional[str | int] = Field(default=None, title='编码')
    description: Optional[str] = Field(default=None, title='链接描述', alias='desc')
    image: Optional[str] = Field(default=None, title='链接图片')
    icon: Optional[str] = Field(default=None, title='链接图标')
    is_active: Optional[bool] = Field(title='是否激活', default=False)
    children: Optional[list['LinkItem']] = Field(title='子项', default=[])
    data_dict: Optional[dict[str, str | int | float]] = Field(title='数据字典', default=dict())



"""
东方财富加载认证的工具类
"""
from afeng_tools.east_money_tool.tool.east_money_http_tools import load_auth_file


def load_stock_comment_auth_file(stock_code: str) -> tuple:
    """
    加载综合评价页面的认证文件: https://data.eastmoney.com/stockcomment/stock/688981.html
    :param stock_code: 股票代码
    :return: 返回结果：cookie_file, header_file
    """
    return load_auth_file(cookie_file_name=f'eastmoney_stock_comment_{stock_code}_cookie.txt',
                          header_file_name=f'eastmoney_stock_comment_{stock_code}_header.txt',
                          refresh_url=f'https://data.eastmoney.com/stockcomment/stock/{stock_code}.html')



"""异步HTTP工具"""
import httpx


class AsyncHttp:
    @classmethod
    async def get(cls, url, callback, params=None, headers=None, cookies=None, proxies=None):
        async with httpx.AsyncClient(headers=headers, cookies=cookies, proxies=proxies) as client:
            resp = await client.get(url, params=params)
            callback(resp)

    @classmethod
    async def post(cls, url, callback, data=None, headers=None, cookies=None, proxies=None):
        async with httpx.AsyncClient(headers=headers, cookies=cookies, proxies=proxies) as client:
            resp = await client.post(url, data=data)
            callback(resp)



import os
import re

from afeng_tools.appium_tool.mobile_keycode import MobileKeycode


def get_devices_info():
    """获取所有连接设备"""
    output = os.popen('adb devices -l')
    device_list = []
    for tmp_line in output.readlines()[1:]:
        if tmp_line.strip():
            tmp_info_list = list(filter(lambda x: x != '', tmp_line.strip().split(' ')))
            device_list.append({
                'id': tmp_info_list[0],
                'product': tmp_info_list[2].split(':')[1],
                'model': tmp_info_list[3].split(':')[1],
                'device': tmp_info_list[4].split(':')[1],
                'transport_id': tmp_info_list[5].split(':')[1],
            })
    return device_list


def get_device_name():
    """获取第一个连接设备的名称"""
    devices_list = get_devices_info()
    if devices_list:
        return devices_list[0]['product']


def get_platform_version():
    """获取安卓版本"""
    output = os.popen("adb shell getprop ro.build.version.release").read()
    if output:
        return output.strip()


def get_current_activity() -> tuple:
    """
    获取当前Activity
    :return: (app_package, app_activity)
    使用示例：app_package, app_activity = get_current_activity()
    """
    output = os.popen('adb shell dumpsys activity | findstr "mResume"').read()
    if output:
        activity_record_obj_str = output.strip().split(':')[1].strip()
        re_match = re.match('^ActivityRecord\\{\w* \w* (\S*)/(\S*) \w*\\}$', activity_record_obj_str)
        if re_match:
            return re_match.group(1), re_match.group(2)


def send_keys(text):
    """输入文本"""
    os.popen(f'adb shell input text {text}').read()


def swipe(start_x, start_y, end_x, end_y) -> None:
    """
    滑动屏幕
    :param start_x: 开始点x坐标
    :param start_y: 开始点y坐标
    :param end_x: 结束点x坐标
    :param end_y: 结束点y坐标
    :return: None
    """
    os.popen(f'adb shell input swipe {start_x} {start_y} {end_x} {end_y}').read()


def click(x, y) -> None:
    """
    点击屏幕
    :param x: x坐标
    :param y: y坐标
    :return: None
    """
    os.popen(f'adb shell input tap {x} {y}').read()


def press_key(mobile_key: MobileKeycode):
    os.popen(f'adb shell input keyevent {mobile_key.value}').read()


def open_tcpip_port(port: int):
    """打开tcpip端口"""
    os.popen(f'adb tcpip {port}')


def connect_device(ip: str, port: int):
    """连接设备"""
    os.popen(f'adb connect {ip}:{port}')


def open_app(app_package: str, app_activity: str):
    """
    打开app
    :param app_package: 要打开的应用包名
    :param app_activity: 要打开的Activity
    :return:
    """
    os.popen(f'adb shell am start -n {app_package}/{app_activity}')


def get_point():
    """获取手机点击坐标"""
    output = os.popen(f'adb shell getevent -p').read()
    print(output)


def get_press_key():
    """查看点击按键"""
    output = os.popen(f'adb shell getevent -t').read()
    print(output)


if __name__ == '__main__':
    print(get_device_name())  # PCT-AL10
    # app_package, app_activity = get_current_activity()
    # print(app_package)
    # print(app_activity)
    print(get_platform_version())
    # press_key(MobileKeycode.KEYCODE_BACK)
    # get_point()
    pass



"""
时间监视器工具
"""
import time
from typing import Callable


def time_monitor(func):
    """装饰器：时间监视器，用于统计执行消耗时间"""

    def func_wrapper(*args, **kwargs):
        start_time = time.perf_counter()
        if isinstance(func, Callable):
            result = func(*args, **kwargs)
        else:
            result = func.__func__(*args, **kwargs)
        end_time = time.perf_counter()
        execution_time = end_time - start_time
        print(f'[{func.__name__}]-[use time ：{execution_time}]')
        return result

    return func_wrapper



"""
FastAPI设置
"""

from typing import Callable

from afeng_tools.fastapi_tool.core.fastapi_enums import FastapiConfigKeyEnum

__FASTAPI_CONFIG_CACHE__ = dict()

from afeng_tools.fastapi_tool.core.fastapi_items import FastapiConfigItem


def init_config(fastapi_config_item: FastapiConfigItem):
    """初始化配置"""
    _set_config(FastapiConfigKeyEnum.is_json_api, fastapi_config_item.is_json_api)
    _set_config(FastapiConfigKeyEnum.error404_context_data_func, fastapi_config_item.error404_context_data_func)
    _set_config(FastapiConfigKeyEnum.error500_context_data_func, fastapi_config_item.error500_context_data_func)
    _set_config(FastapiConfigKeyEnum.error501_context_data_func, fastapi_config_item.error501_context_data_func)
    _set_config(FastapiConfigKeyEnum.error500_background_work_func, fastapi_config_item.error500_background_work_func)


def _set_config(config_key: FastapiConfigKeyEnum, config_value: str | Callable | bool):
    """设置配置"""
    __FASTAPI_CONFIG_CACHE__[config_key] = config_value


def get_config(config_key: FastapiConfigKeyEnum) -> str | Callable:
    """获取配置"""
    return __FASTAPI_CONFIG_CACHE__.get(config_key)



from typing import Optional

from pydantic import BaseModel


class DatabaseInfoItem(BaseModel):
    """数据库信息项"""
    # 数据库连接地址
    database_uri: str
    # 是否打印sql
    echo_sql: Optional[bool] = False
    # 数据库连接池
    pool_size: Optional[int] = 0
    # 是否自动提交
    auto_commit: Optional[bool] = False
    # 是否自动flush
    auto_flush: Optional[bool] = False
    # 超时是否自动提交
    expire_on_commit: Optional[bool] = False
    # 预ping检查
    pool_pre_ping: Optional[bool] = True
    # 回收时间
    pool_recycle: Optional[int] = 0
    # 是否先进先出
    pool_use_lifo: Optional[bool] = True


"""
使用PyQyery操作HTML： http://pyquery.readthedocs.io/en/latest/api.html
    安装：pip install lxml -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
         pip install pyquery -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
from pyquery import PyQuery


def get_PyQuery_from_url(url: str) -> PyQuery:
    """获取PyQuery（通过url）"""
    return PyQuery(url=url)


def get_PyQuery_from_html(html_content: str) -> PyQuery:
    """获取PyQuery（通过html内容）"""
    return PyQuery(html_content)


def get_PyQuery_from_file(html_file: str, encoding: str = "utf-8") -> PyQuery:
    """获取PyQuery（通过html文件）"""
    return PyQuery(filename=html_file, encoding=encoding)


def get_PyQuery(html_content: str = None, url: str = None, html_file: str = None, encoding: str = "utf-8"):
    """获取PyQuery"""
    if url is not None:
        return get_PyQuery_from_url(url)
    elif html_file is not None:
        return get_PyQuery_from_file(html_file, encoding=encoding)
    else:
        return get_PyQuery_from_html(html_content)


def query(dom: PyQuery, css_selector) -> PyQuery:
    """通过[css选择器/伪类选择器]查询元素"""
    return dom(css_selector)


def find(el: PyQuery, css_selector) -> PyQuery:
    """通过[css选择器/伪类选择器]查找当前节点的子节点"""
    return el.find(css_selector)


def children(el: PyQuery, css_selector: str = None) -> PyQuery:
    """通过[css选择器/伪类选择器]获取当前节点的直接子节点"""
    if css_selector is None:
        return el.children()
    return el.children(css_selector)


def parent(el: PyQuery) -> PyQuery:
    """通过[css选择器/伪类选择器]获取当前节点的直接父亲节点"""
    return el.parent()


def parents(el: PyQuery, css_selector: str = None) -> PyQuery:
    """通过[css选择器/伪类选择器]获取当前节点的所有祖宗节点"""
    if css_selector is None:
        return el.parents()
    return el.parents(css_selector)


def siblings(el: PyQuery, css_selector: str = None) -> PyQuery:
    """通过[css选择器/伪类选择器]获取当前节点的兄弟节点"""
    if css_selector is None:
        return el.siblings()
    return el.siblings(css_selector)


def set_html(el: PyQuery, value: str) -> PyQuery:
    """获取元素的html内容"""
    return el.html(value)


def get_html(el: PyQuery) -> str:
    """获取元素的html内容"""
    return el.html()


def set_text(el: PyQuery, value: str) -> PyQuery:
    """获取元素的text文本"""
    return el.text(value)


def get_text(el: PyQuery) -> str:
    """获取元素的text文本"""
    return el.text()


def get_attr(el: PyQuery, attr_name: str) -> str:
    """获取元素的属性值"""
    return el.attr(attr_name)


def set_attr(el: PyQuery, attr_name: str, attr_value: str) -> PyQuery:
    """设置元素的属性值"""
    return el.attr(attr_name, attr_value)


def get_items(el: PyQuery) -> list[PyQuery]:
    """获取元素列表项"""
    return el.items()


def add_class(el: PyQuery, class_value) -> PyQuery:
    """添加class"""
    return el.add_class(class_value)


def remove_class(el: PyQuery, class_value) -> PyQuery:
    """移除class"""
    return el.remove_class(class_value)


def add_style(el: PyQuery, css_field: str, css_value: str) -> PyQuery:
    """
    添加style样式
    :param el:
    :param css_field: 如：‘font-size’
    :param css_value: 如：14px
    :return: PyQuery
    """
    return el.css(css_field, css_value)


def remove_el(el: PyQuery):
    """移除元素"""
    return el.remove()



"""
装饰器工具
"""
from typing import Callable, Any


def run_func(func: Callable, *args, **kwargs):
    """运行函数"""
    if isinstance(func, Callable):
        return func(*args, **kwargs)
    else:
        return func.__func__(*args, **kwargs)


async def run_async_func(func: Callable, *args, **kwargs):
    """运行异步函数"""
    if isinstance(func, Callable):
        return await func(*args, **kwargs)
    else:
        return await func.__func__(*args, **kwargs)


def append_decorator_info(func: Callable, decorator_name: str):
    """添加装饰器信息（装饰器中内部函数需要有装饰器：@wraps(func)）"""
    if hasattr(func, '__decorators__'):
        getattr(func, '__decorators__').append(decorator_name)
    else:
        setattr(func, '__decorators__', [decorator_name])


def append_func_info(func: Callable, key: str, value: Any):
    """通过装饰器添加函数信息（装饰器中内部函数需要有装饰器：@wraps(func)）"""
    setattr(func, key, value)


def get_func_by_decorator(module, decorator_name) -> dict[str, Callable]:
    """获取模块中具有某个装饰器的函数列表"""
    func_members = {}
    for tmp_key in dir(module):
        if tmp_key.startswith('__'):
            continue
        tmp_value = module.__dict__[tmp_key]
        if isinstance(tmp_value, Callable) and hasattr(tmp_value, '__decorators__') and decorator_name in getattr(tmp_value, '__decorators__'):
            func_members[tmp_key] = tmp_value
    return func_members



"""
数字工具：
1、可以使用模块：pip install cn2an
    将字符串中的数字转为汉字： cn2an.transform(str_data, "an2cn")

"""
from afeng_tools.math_tool.core import number_to_chinese


def num_to_chinese(num_value: int):
    """数字转中文"""
    try:
        import cn2an
        return number_to_chinese.num_to_chinese(num_value)
    except Exception:
        return number_to_chinese.to_chinese(num_value)



"""
页面：https://data.eastmoney.com/stockcomment/stock/688981.html
主力控盘
"""
from afeng_tools.east_money_tool.tool.east_money_http_tools import get_datacenter_data
from afeng_tools.east_money_tool.tool.east_money_load_auth_tools import load_stock_comment_auth_file


def load_jgcy(stock_code: str, page_size: int = 22):
    """加载机构参与程度-折线图数据"""
    cookie_file, header_file = load_stock_comment_auth_file(stock_code)
    response_data = get_datacenter_data(stock_code=stock_code,
                                        opt_dict={'reportName': 'RPT_DMSK_TS_STOCKEVALUATE',
                                       'sortColumns': "TRADE_DATE",
                                       'sortTypes': "-1",
                                       'pageSize': page_size
                                       },
                                        cookie_file=cookie_file,
                                        header_file=header_file)
    if response_data['success'] and response_data['result']:
        result = response_data['result']
        total_count = result['count']
        # total_pages = result['pages']
        # if total_count > page_size:
        #     return load_jgcy(stock_code=stock_code, page_size=total_count)
        return result['data']


if __name__ == '__main__':
    load_jgcy(stock_code='000001')



from pydantic.main import BaseModel


def format_xml_msg(model: BaseModel):
    """使用xml文件格式化消息"""
    xml_file = getattr(model, '_xml_file')
    with open(xml_file, 'r', encoding='utf-8') as f:
        xml_format = f.read()
        return xml_format.format(**model.dict())



import os
from enum import Enum
from typing import Callable, Optional, Any

from pydantic import BaseModel

from afeng_tools.os_tool.os_tools import get_user_home
from afeng_tools.weixin_tool.core.model.item.wx_receive_event_models import WeixinEventItem, WeixinSubscribeEventItem, \
    WeixinQrEventItem, WeixinScanEventItem, WeixinLocationEventItem, WeixinMenuEventItem
from afeng_tools.weixin_tool.core.model.item.wx_receive_msg_models import WeixinVoiceMsgItem, WeixinTextMsgItem, \
    WeixinImageMsgItem, WeixinVideoMsgItem, WeixinLocationMsgItem, WeixinLinkMsgItem
from afeng_tools.weixin_tool.core.response import XmlResponse


class WeixinConfigItem(BaseModel):
    """微信配置枚举"""
    weixin_app_id: Optional[str] = None
    weixin_app_secret: Optional[str] = None
    weixin_token: Optional[str] = None
    weixin_encoding_aes_key: Optional[str] = None
    weixin_msg_callback: Optional[Callable[[
        WeixinTextMsgItem | WeixinImageMsgItem | WeixinVoiceMsgItem | WeixinVideoMsgItem | WeixinLocationMsgItem | WeixinLinkMsgItem | WeixinEventItem | WeixinSubscribeEventItem | WeixinQrEventItem | WeixinScanEventItem | WeixinLocationEventItem | WeixinMenuEventItem, bytes, dict[str, Any]], XmlResponse]] = None
    weixin_token_file: Optional[str] = os.path.join(get_user_home(), f'.wx_access_token.bin')


class WeixinConfigKeyEnum(Enum):
    """微信配置枚举"""
    # 微信公众号app_id
    weixin_app_id = 'weixin_app_id'
    # 微信公众号app_secret
    weixin_app_secret = 'weixin_app_secret'
    # 微信公众号token
    weixin_token = 'weixin_token'
    # 微信公众号token
    weixin_encoding_aes_key = 'weixin_encoding_aes_key'
    # 微信公众号接到消息后的回调函数两个参数(msg_model, msg_bytes:bytes)
    weixin_msg_callback = 'weixin_msg_callback'
    # 微信公众号token文件的存储路径
    weixin_token_file = 'weixin_token_file'




# from sqlalchemy import Column, Integer, String
#
# from afeng_tools.sqlalchemy_tool.core.sqlalchemy_base_model import Model
#
#
# class WeixinMsgBaseModel(Model):
#     """微信消息基本信息表"""
#     __abstract__ = True
#     openid = Column(String(125), comment='OpenID', nullable=False)
#     to_user = Column(String(250), comment='开发者微信号', nullable=False)
#     from_user = Column(String(250), comment='发送方帐号（一个OpenID）', nullable=False)
#     create_time = Column(Integer, comment='消息创建时间 （整型）', nullable=False)
#     # 消息类型，文本为text, 图片为image，语音为voice，视频为video,小视频为shortvideo，地理位置为location，链接为link
#     msg_type = Column(String(30), comment='消息类型', nullable=False)
#     msg_id = Column(String(64), comment='消息id，64位整型', nullable=False)
#     msg_data_id = Column(String(64), comment='消息的数据ID（消息如果来自文章时才有）', nullable=True)
#     idx = Column(String(64), comment='多图文时第几篇文章，从1开始（消息如果来自文章时才有）', nullable=True)
#
#
# class WeixinTextMsg(WeixinMsgBaseModel):
#     """文本消息表"""
#     __tablename__ = 'weixin_text_msg'
#     msg_content = Column(String(850), comment='文本消息内容', nullable=False)
#
#     def __repr__(self):
#         return self.msg_content[:50]
#
#
# class WeixinImageMsg(WeixinMsgBaseModel):
#     """图片消息表"""
#     __tablename__ = 'weixin_image_msg'
#     pic_url = Column(String(500), comment='图片链接（由系统生成）', nullable=False)
#     media_id = Column(String(250), comment='图片消息媒体id，可以调用获取临时素材接口拉取数据。', nullable=False)
#
#     def __repr__(self):
#         return self.pic_url
#
#
# class WeixinVoiceMsg(WeixinMsgBaseModel):
#     """语音消息表"""
#     __tablename__ = 'weixin_voice_msg'
#     media_id = Column(String(250), comment='语音消息媒体id，可以调用获取临时素材接口拉取数据。', nullable=False)
#     voice_format = Column(String(30), comment='语音格式，如amr，speex等', nullable=False)
#     recognition = Column(String(500), comment='语音识别结果，UTF8编码', nullable=True)
#
#     def __repr__(self):
#         return self.media_id
#
#
# class WeixinVideoMsg(WeixinMsgBaseModel):
#     """视频消息表"""
#     __tablename__ = 'weixin_video_msg'
#     media_id = Column(String(250), comment='视频消息媒体id，可以调用获取临时素材接口拉取数据。', nullable=False)
#     thumb_media_id = Column(String(255), comment='视频消息缩略图的媒体id，可以调用多媒体文件下载接口拉取数据。',
#                             nullable=False)
#
#     def __repr__(self):
#         return self.media_id
#
#
# class WeixinLocationMsg(WeixinMsgBaseModel):
#     """地理位置消息表"""
#     __tablename__ = 'weixin_location_msg'
#     location_x = Column(String(20), comment='地理位置纬度', nullable=False)
#     location_y = Column(String(20), comment='地理位置经度', nullable=False)
#     scale = Column(String(10), comment='地图缩放大小', nullable=False)
#     label = Column(String(150), comment='地理位置信息', nullable=False)
#
#     def __repr__(self):
#         return self.label
#
#
# class WeixinLinkMsg(WeixinMsgBaseModel):
#     """链接消息表"""
#     __tablename__ = 'weixin_link_msg'
#     title = Column(String(100), comment='消息标题', nullable=False)
#     description = Column(String(500), comment='消息描述', nullable=False)
#     url = Column(String(500), comment='消息链接', nullable=False)
#
#     def __repr__(self):
#         return self.title



"""
less工具：pip install lesscpy -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
- https://pypi.org/project/lesscpy/
"""
import os

import lesscpy
from io import StringIO


def compile_less(less_str: str, minify: bool = True):
    return lesscpy.compile(StringIO(less_str), minify=minify)


def compile_to_css(less_code: str, less_dir: str = None, minify: bool = True):
    """
    less转css
    :param less_code: less代码
    :param less_dir: less文件所在目录
    :param minify: 是否压缩
    :return: css代码
    """
    old_work_dir = os.getcwd()
    if less_dir and '@import ' in less_code:
        # 改变当前路径为less的路径
        os.chdir(less_dir)
    result = lesscpy.compile(StringIO(less_code), minify=minify)
    os.chdir(old_work_dir)
    return result
